"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const ts = require("typescript");
const common_1 = require("../../common");
function getModuleFile(tree, options) {
    const modulePath = options.module;
    if (!tree.exists(modulePath)) {
        throw new schematics_1.SchematicsException(`File ${modulePath} does not exist.`);
    }
    const text = tree.read(modulePath);
    if (text === null) {
        throw new schematics_1.SchematicsException(`File ${modulePath} does not exist.`);
    }
    const sourceText = text.toString('utf-8');
    return ts.createSourceFile(modulePath, sourceText, ts.ScriptTarget.Latest, true);
}
function applyChanges(tree, path, changes) {
    const recorder = tree.beginUpdate(path);
    for (const change of changes) {
        recorder.insertLeft(change.pos, change.toAdd);
    }
    tree.commitUpdate(recorder);
    return tree;
}
function addImportsToModuleFile(options, imports, importName = common_1.packageName) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = ast_utils_1.insertImport(module, options.module, imports.join(', '), importName);
        return applyChanges(tree, options.module, [
            importChanges,
        ]);
    };
}
function addImportsToModuleDeclaration(options, imports) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = imports.map((imp) => ast_utils_1.addImportToModule(module, options.module, imp, common_1.packageName)[0]);
        return applyChanges(tree, options.module, importChanges);
    };
}
function ngAdd(options) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const project = yield common_1.getProject(tree, options.project);
        const sourceRoot = (_a = (project && project.sourceRoot)) !== null && _a !== void 0 ? _a : 'src';
        const modulesToAdd = ['LetModule', 'PushModule'];
        options.module = common_1.findRootModule(tree, options.module, sourceRoot);
        return schematics_1.chain([
            addImportsToModuleFile(options, modulesToAdd),
            addImportsToModuleDeclaration(options, modulesToAdd),
        ]);
    });
}
exports.ngAdd = ngAdd;
